@@ -277,6 +277,7 @@ NoRedeemer = Nothing
 def all_tokens_unlocked_from_address(
     txins: List[TxInInfo], address: Address, token: Token
 ) -> int:
+    """Returns how many tokens of specified type are unlocked from given address"""
     return sum(
         [
             txi.resolved.value.get(token.policy_id, {b"": 0}).get(token.token_name, 0)
@@ -289,6 +290,7 @@ def all_tokens_unlocked_from_address(
 def all_tokens_locked_at_address_with_datum(
     txouts: List[TxOut], address: Address, token: Token, output_datum: OutputDatum
 ) -> int:
+    """Returns how many tokens of specified type are locked at then given address with the specified datum"""
     return sum(
         [
             txo.value.get(token.policy_id, {b"": 0}).get(token.token_name, 0)
@@ -301,6 +303,7 @@ def all_tokens_locked_at_address_with_datum(
 def all_tokens_locked_at_address(
     txouts: List[TxOut], address: Address, token: Token
 ) -> int:
+    """Returns how many tokens of specified type are locked at then given address"""
     return sum(
         [
             txo.value.get(token.policy_id, {b"": 0}).get(token.token_name, 0)
@@ -308,3 +311,8 @@ def all_tokens_locked_at_address(
             if txo.address == address
         ]
     )
+
+
+def resolve_spent_utxo(txins: List[TxInInfo], p: Spending) -> TxOut:
+    """Returns the UTxO whose spending should be validated"""
+    return [txi.resolved for txi in txins if txi.out_ref == p.tx_out_ref][0]
